import React, { FC, useEffect, useState } from 'react';
import { createMuiTheme, ThemeProvider } from '@material-ui/core/styles';
import { BrowserRouter as Router, Switch, Route } from 'react-router-dom';
import { HomePage } from 'components/AuthUser/HomePage/HomePage';
import { AuthUser } from 'components/AuthUser/AuthUser';
import { LandingPage } from 'components/NonAuthUser/LandingPage/LandingPage';
import { PrivateRoute } from 'api/PrivateRoute/PrivateRoute';
import { AuthRoutes, NonAuthRoutes } from 'api/routes';
import { NotFound } from 'components/NonAuthUser/NotFound/NotFound';
import { ProfilePage } from 'components/AuthUser/ProfilePage/ProfilePage';
import { Roles } from 'api/userRoles';
import { Unauthorized } from 'components/NonAuthUser/Unauthorized/Unauthorized';
import axios from 'axios';
import { ReservationPage } from 'components/AuthUser/ReservationPage/ReservationPage';

const theme = createMuiTheme({
  palette: {
    primary: {
      main: '#5e5ce4',
    },
    secondary: {
      main: '#e2e45c',
    },
  },

  typography: {
    fontSize: 16,
  },

  overrides: {
    MuiTabs: {
      indicator: {
        backgroundColor: 'white',
      },
    },
    MuiTab: {
      wrapper: {
        flexDirection: 'row',
      },
    },
  },
});

const configDjangoCookieName = (): void => {
  axios.defaults.xsrfHeaderName = 'X-CSRFTOKEN';
  axios.defaults.xsrfCookieName = 'csrftoken';
  axios.defaults.withCredentials = true;
};

export const App: FC = () => {
  const [isCookieFetched, setisCookieFetched] = useState<string>('');
  configDjangoCookieName();
  useEffect(() => {
    const fetchCookie = async (): Promise<unknown> => {
      const response = await axios('api/web/csrf');
      axios.defaults.headers.common['X-CSRFTOKEN'] = response.data.token;
      sessionStorage.setItem('X-CSRFTOKEN', response.data.token);
      sessionStorage.setItem('ROLE', 'admin');
      setisCookieFetched(response.data.token);
      return null;
    };
    if (!isCookieFetched) fetchCookie();
  }, [isCookieFetched]);

  return (
    <ThemeProvider theme={theme}>
      <Router>
        <div data-testid="App">
          <Switch>
            <Route path={NonAuthRoutes.auth} component={AuthUser} />
            <Route exact path={NonAuthRoutes.home} component={LandingPage} />
            <PrivateRoute
              path={AuthRoutes.dashboard}
              Component={HomePage}
              requiredRoles={[Roles.admin, Roles.operator, Roles.senior]}
            />
            <PrivateRoute
              path={AuthRoutes.profile}
              Component={ProfilePage}
              requiredRoles={[Roles.admin, Roles.operator, Roles.senior]}
            />
            <Route path={AuthRoutes.reservation} component={ReservationPage} />
            <Route path={NonAuthRoutes.unauthorized} component={Unauthorized} />
            <Route component={NotFound} />
          </Switch>
        </div>
      </Router>
    </ThemeProvider>
  );
};
